@@ -23,6 +23,7 @@
 import dataiter
 import datetime
 import numpy as np
+import sys
 
 from dataiter import util
 
@@ -105,7 +106,7 @@ class Vector(np.ndarray):
         >>> vector.as_bytes()
         """
         if self.is_string():
-            array = np.char.encode(self, "UTF-8")
+            array = np.char.encode(self, "utf-8")
             return array.view(self.__class__)
         return self.astype(bytes)
 
@@ -204,6 +205,17 @@ class Vector(np.ndarray):
             object = list(object)
         return np.array(object, dtype).view(cls)
 
+    def get_memory_use(self):
+        """
+        Return memory use in bytes.
+
+        >>> vector = di.Vector(range(100))
+        >>> vector.get_memory_use()
+        """
+        if self.is_object():
+            return sum(sys.getsizeof(x) for x in self)
+        return self.nbytes
+
     def head(self, n=None):
         """
         Return the first `n` elements.
@@ -333,9 +345,9 @@ class Vector(np.ndarray):
         """
         Return the corresponding value to use to represent missing data.
 
-        dataiter is built on top of NumPy. NumPy doesn't support a proper
+        Dataiter is built on top of NumPy. NumPy doesn't support a proper
         missing value ("NA"), only two data type specific values: ``np.nan``
-        and ``np.datetime64("NaT")``. dataiter recommends the following values
+        and ``np.datetime64("NaT")``. Dataiter recommends the following values
         be used and internally supports them to an extent.
 
         ======== ========================
@@ -372,17 +384,58 @@ class Vector(np.ndarray):
         rng = [np.nanmin(self), np.nanmax(self)]
         return self.__class__(rng, self.dtype)
 
-    def rank(self):
+    def rank(self, method="average"):
         """
         Return the order of elements in a sorted vector.
 
-        Note that these are not unique indices as ties result in duplicates.
+        `method` determines how ties are resolved. **'min'** assigns each of
+        equal values the same rank, the minimum of the set (also called
+        "competition ranking"). **'max'** is the same, but assigning the
+        maximum of the set. **'average'** is the mean of 'min' and 'max'.
+        **'ordinal'** gives each element a distinct rank with equal values
+        ranked by their order in input.
 
-        >>> vector = di.Vector([1, 2, 1, 2, 3])
-        >>> vector.rank()
+        Ranks begin at 1. Missing values are ranked last.
+
+        **References**
+
+        * https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.rankdata.html
+        * https://www.rdocumentation.org/packages/base/topics/rank
+
+        >>> vector = di.Vector([3, 1, 1, 1, 2, 2])
+        >>> vector.rank(method="min")
+        >>> vector.rank(method="max")
+        >>> vector.rank(method="average")
+        >>> vector.rank(method="ordinal")
         """
-        rank = np.unique(self, return_inverse=True)[1]
-        return rank.view(self.__class__)
+        if method not in ["min", "max", "average", "ordinal"]:
+            raise ValueError(f"Unexpected method: {method!r}")
+        missing = self.is_missing()
+        if method == "average":
+            rank_min = self.rank(method="min")
+            rank_max = self.rank(method="max")
+            rank = np.mean([rank_min, rank_max], axis=0)
+            return self.__class__(rank)
+        if method == "min":
+            # https://stackoverflow.com/a/14672797/16369038
+            inv = np.unique(self[~missing], return_inverse=True)[1]
+            arank = np.concatenate(([0], np.bincount(inv))).cumsum()[inv]
+            zrank = arank.max() + 1
+        if method == "max":
+            # https://stackoverflow.com/a/14672797/16369038
+            inv = np.unique(self[~missing], return_inverse=True)[1]
+            arank = np.bincount(inv).cumsum()[inv] - 1
+            zrank = len(self) - 1
+        if method == "ordinal":
+            # https://stackoverflow.com/a/5284703/16369038
+            indices = self[~missing].argsort()
+            arank = np.empty_like(indices)
+            arank[indices] = np.arange(len(indices))
+            zrank = arank.max() + 1 + np.arange(missing.sum())
+        out = np.zeros_like(self, int)
+        out[~missing] = arank + 1
+        out[missing] = zrank + 1
+        return self.__class__(out)
 
     def replace_missing(self, value):
         """
@@ -420,25 +473,38 @@ class Vector(np.ndarray):
         >>> vector.sort(dir=1)
         >>> vector.sort(dir=-1)
         """
-        vector = self.copy()
-        np.ndarray.sort(vector)
+        missing = self.is_missing()
+        a = self[~missing].copy()
+        z = self[missing].copy()
+        np.ndarray.sort(a)
         if dir < 0:
-            # Flip order, but keep missing last.
-            na = vector.is_missing()
-            ok = np.nonzero(~na)
-            np.put(vector, ok, vector[ok][::-1])
-        return vector
+            a = a[::-1]
+        vector = np.concatenate((a, z))
+        return self.__class__(vector)
 
     @classmethod
     def _std_to_np(cls, seq, dtype=None):
         # Convert missing values in seq to NumPy equivalents.
+        # Can be empty if all of seq are missing values.
         types = util.unique_types(seq)
-        missing = cls._std_to_np_missing_value(types)
+        if dtype is not None:
+            missing = Vector.fast([], dtype).missing_value
+        elif len(types) == 1 and types.copy().pop().__module__ == "numpy":
+            # If we have a regular Python list of NumPy scalars,
+            # infer type. This should be rare, but can happen.
+            dtype = types.copy().pop()().dtype
+            missing = Vector.fast([], dtype).missing_value
+        else:
+            # Guess the missing value based on types in seq.
+            missing = cls._std_to_np_missing_value(types)
         seq = [missing if
                x is None or
                (isinstance(x, float) and np.isnan(x))
                else x for x in seq]
         if dtype is not None:
+            if np.issubdtype(dtype, np.integer) and np.nan in seq:
+                # Upcast from integer to float as required.
+                dtype = float
             return np.array(seq, dtype)
         # NaT values bring in np.datetime64 to types.
         types.discard(np.datetime64)
@@ -450,6 +516,8 @@ class Vector(np.ndarray):
 
     @classmethod
     def _std_to_np_missing_value(cls, types):
+        if not types:
+            return None
         if str in types:
             return ""
         if all(x in [float, int] for x in types):
